searchData={"content_type":"text/markdown","items":[{"doc":"Autoslug field for Ecto.","ref":"EctoAutoslugField.html","title":"EctoAutoslugField","type":"module"},{"doc":"This module defines all the required functions and modules to work with.","ref":"EctoAutoslugField.Slug.html","title":"EctoAutoslugField.Slug","type":"module"},{"doc":"To create a simple 'Slug' field do:\n\n    defmodule MyCustomSlug do\n      use EctoAutoslugField.Slug, from: :name_field, to: :slug_field\n    end\n\nIt is also possible to override `get_sources/2` and `build_slug/2` functions\nwhich are part of the AutoslugField's API.\n\nMore complex example with the optional sources\nand custom slug generation function:\n\n    defmodule MyComplexSlug do\n      use EctoAutoslugField.Slug, to: :slug_field\n\n      def get_sources(changeset, _opts) do\n        basic_fields = [:name, :surname]\n\n        if is_company_info_set(changeset) do\n          # We want to track changes in the person's company:\n          basic_fields ++ [:company, :position]\n        else\n          basic_fields\n        end\n      end\n\n      def build_slug(sources, changeset) do\n        super(sources, changeset)  # Calls the `SlugGenerator.build_slug/1`\n        |> String.replace(\"-\", \"+\")\n      end\n    end\n\nIt is also possible to always change your slug, even if it was already set:\n\n    defmodule ThisSlugShouldChange do\n      use EctoAutoslugField.Slug, from: :some_field,\n        to: :slug_field, always_change: true\n    end\n\nIf you want to change slug for only one instance\nwithout setting `always_change` option, use `force_generate_slug/1` function:\n\n    defmodule SimpleSlugForce do\n      use EctoAutoslugField.Slug, from: :name, to: :some_slug\n    end\n\nThen you can use `SimpleSlugForce.force_generate_slug(changeset)`\nfor any instances, that needs to recreate slugs for some reason.\n\nBe careful with these options, since\n[cool URIs do not change](https://www.w3.org/Provider/Style/URI.html).","ref":"EctoAutoslugField.Slug.html#module-examples","title":"Examples - EctoAutoslugField.Slug","type":"module"},{"doc":"This module defines all functions `Slug` module uses.","ref":"EctoAutoslugField.SlugBase.html","title":"EctoAutoslugField.SlugBase","type":"module"},{"doc":"This function is used to build the slug itself.\n\nThis function is a place to modify the result slug.\nFor convenience you can call `super(sources, changeset)`\nwhich will return the slug binary.\n`super(sources)` uses [`slugify`](https://github.com/jayjun/slugify),\nbut you can completely change slug-engine to your own.\n\nNote: this function will only be called if `sources` is not empty.\nAlso important this function will be called only\nonce for the normal workflow. And every time for `:always_change`.\nSo you can do some heavy computations.\n\nIf for some reason slug should not be set -\njust return `nil` or empty `binary`.\n\nIt should return a `binary` or `nil`.","ref":"EctoAutoslugField.SlugBase.html#build_slug/2","title":"EctoAutoslugField.SlugBase.build_slug/2","type":"function"},{"doc":"This function is used to get sources for the slug.\n\nThere can be many use cases when this behavior is required,\nhere are the brief examples:\n\n  1. Conditional slug sources\n  2. Add any data from different sources\n\nThis function should return `list` of atoms or binaries, or `nil`.\n\nWhen processing the returned list:\n\n  1. `atom`-key is supposed to identify the model field\n  2. `binary`-key is treated as a data itself, it won't be changed","ref":"EctoAutoslugField.SlugBase.html#get_sources/2","title":"EctoAutoslugField.SlugBase.get_sources/2","type":"function"},{"doc":"This function is used to generate slug.\n\nIt is called 'maybe' since it may not generate slug for several reasons:\n\n  1. It was already created and `:always_change` option was not set\n  2. The source fields for the slug were empty\n\nThis function takes changeset as an input and returns changeset.","ref":"EctoAutoslugField.SlugBase.html#maybe_generate_slug/3","title":"EctoAutoslugField.SlugBase.maybe_generate_slug/3","type":"function"},{"doc":"This is just a helper function to check for uniqueness.\n\nIt basically just wraps `Ecto.Changeset` and set a proper field name.","ref":"EctoAutoslugField.SlugBase.html#unique_constraint/3","title":"EctoAutoslugField.SlugBase.unique_constraint/3","type":"function"},{"doc":"This module works with slugs itself. It is just a wrapper around 'Slugify'.\n\nIt is suited for inner use.","ref":"EctoAutoslugField.SlugGenerator.html","title":"EctoAutoslugField.SlugGenerator","type":"module"},{"doc":"This is a public wrapper around `do_build_slug/1` functions.\n\nDefault slug builder.","ref":"EctoAutoslugField.SlugGenerator.html#build_slug/2","title":"EctoAutoslugField.SlugGenerator.build_slug/2","type":"function"},{"doc":"This function conditionally generates slug.\n\nThis function prepares sources and then calls `do_generate_slug/3`.","ref":"EctoAutoslugField.SlugGenerator.html#maybe_generate_slug/3","title":"EctoAutoslugField.SlugGenerator.maybe_generate_slug/3","type":"function"},{"doc":"This module represents a simple wrapper around ':string' Ecto type.","ref":"EctoAutoslugField.Type.html","title":"EctoAutoslugField.Type","type":"module"},{"doc":"","ref":"EctoAutoslugField.Type.html#cast/1","title":"EctoAutoslugField.Type.cast/1","type":"function"},{"doc":"","ref":"EctoAutoslugField.Type.html#dump/1","title":"EctoAutoslugField.Type.dump/1","type":"function"},{"doc":"","ref":"EctoAutoslugField.Type.html#embed_as/1","title":"EctoAutoslugField.Type.embed_as/1","type":"function"},{"doc":"","ref":"EctoAutoslugField.Type.html#equal?/2","title":"EctoAutoslugField.Type.equal?/2","type":"function"},{"doc":"","ref":"EctoAutoslugField.Type.html#load/1","title":"EctoAutoslugField.Type.load/1","type":"function"},{"doc":"","ref":"EctoAutoslugField.Type.html#type/0","title":"EctoAutoslugField.Type.type/0","type":"function"},{"doc":"# Changelog\n\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](https://keepachangelog.com/en/1.0.0/),\nand this project adheres to [Semantic Versioning](https://semver.org/spec/v2.0.0.html).","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"#","ref":"changelog.html#v3-1-0-2023-08-10","title":"v3.1.0 - 2023-08-10 - Changelog","type":"extras"},{"doc":"- Now supporting `elixir>=1.13`\n- Now using `slugify` instead of `slugger`","ref":"changelog.html#features","title":"Features - Changelog","type":"extras"},{"doc":"#","ref":"changelog.html#v3-0-0-2021-09-12","title":"v3.0.0 - 2021-09-12 - Changelog","type":"extras"},{"doc":"- Now support `elixir>=1.10`\n- Updates `ecto` to `3.7`\n- Updates bunch of other deps to newer versions\n\n#","ref":"changelog.html#features","title":"Features - Changelog","type":"extras"},{"doc":"- Moves from Travis to GitHub Actions\n- Adds `mix format` support","ref":"changelog.html#misc","title":"Misc - Changelog","type":"extras"},{"doc":"#","ref":"changelog.html#v2-0-1-2019-10-16","title":"v2.0.1 - 2019-10-16 - Changelog","type":"extras"},{"doc":"- Fixed warnings with `ecto>=3.2`, see [#101](https://github.com/sobolevn/ecto_autoslug_field/pull/101)","ref":"changelog.html#bugfixes","title":"Bugfixes - Changelog","type":"extras"},{"doc":"Breaking features:\n\n- Now supports `elixir>=1.6`\n- Now support only `ecto>=3.1`\n\n#","ref":"changelog.html#v2-0-0-2019-07-08","title":"v2.0.0 - 2019-07-08 - Changelog","type":"extras"},{"doc":"- Adds `mix format`","ref":"changelog.html#improvements","title":"Improvements - Changelog","type":"extras"},{"doc":"#","ref":"changelog.html#v1-0-0-2018-12-28","title":"v1.0.0 - 2018-12-28 - Changelog","type":"extras"},{"doc":"- Adds support for `ecto >= 3.0`\n- Adds official support for `elixir == 1.6` and `elixir == 1.7`","ref":"changelog.html#improvements","title":"Improvements - Changelog","type":"extras"},{"doc":"#","ref":"changelog.html#v0-5-1-2018-05-21","title":"v0.5.1 - 2018-05-21 - Changelog","type":"extras"},{"doc":"- Updates docs on using `force_update_slug`","ref":"changelog.html#documentation","title":"Documentation - Changelog","type":"extras"},{"doc":"#","ref":"changelog.html#v0-5-0-2018-05-21","title":"v0.5.0 - 2018-05-21 - Changelog","type":"extras"},{"doc":"- Adds `force_generate_slug` function to the client API. This function may be used when the `always_change` option of the slug is maybe not set to `true` but you want to override this setting on the given changeset and force regeneration of the slug\n- Updates multiple dependencies","ref":"changelog.html#improvements","title":"Improvements - Changelog","type":"extras"},{"doc":"#","ref":"changelog.html#v0-4-0-2017-11-23","title":"v0.4.0 - 2017-11-23 - Changelog","type":"extras"},{"doc":"- Adds support for [numeric, datetime and date fields](https://github.com/sobolevn/ecto_autoslug_field/pull/18)\n- Updates dependencies, now supports `ecto >= 2.1`\n\n#","ref":"changelog.html#improvements","title":"Improvements - Changelog","type":"extras"},{"doc":"- Updates `README.md` with the new `ecto` version\n\n#","ref":"changelog.html#documentation","title":"Documentation - Changelog","type":"extras"},{"doc":"- Adds new test cases to cover new field types","ref":"changelog.html#testing","title":"Testing - Changelog","type":"extras"},{"doc":"#","ref":"changelog.html#v0-3-1-2017-07-19","title":"v0.3.1 - 2017-07-19 - Changelog","type":"extras"},{"doc":"- Adds `build_slug/2` to accept the original `changeset` as the second argument, it still receives list of `sources` as the first argument\n- Updates `build_slug/1` inner logic\n\n#","ref":"changelog.html#improvements","title":"Improvements - Changelog","type":"extras"},{"doc":"- Updates `README.md` with the new example\n- Updates docs to handle new changes\n- Updates `CONTRIBUTING.md` with 'Development' section\n\n#","ref":"changelog.html#documentation","title":"Documentation - Changelog","type":"extras"},{"doc":"- Adds new test cases to cover `build_slug/2`","ref":"changelog.html#testing","title":"Testing - Changelog","type":"extras"},{"doc":"#","ref":"changelog.html#v0-3-0-2017-06-11","title":"v0.3.0 - 2017-06-11 - Changelog","type":"extras"},{"doc":"- *breaking* Updates `mix.exs` with new dependencies, closes [#14](https://github.com/sobolevn/ecto_autoslug_field/issues/14)\n- *breaking* Drops `elixir` version 1.2 and `otp` version 17 support\n\n#","ref":"changelog.html#changes","title":"Changes - Changelog","type":"extras"},{"doc":"- Updates `cast/4` functions to `cast/3` functions (new Ecto)\n- Updates `credo` version and `.credo.exs` config\n- Updates `.travis.yml` with credo and new `otp` release\n\n#","ref":"changelog.html#improvements","title":"Improvements - Changelog","type":"extras"},{"doc":"- Fixes dialyxer issues, closes [#10](https://github.com/sobolevn/ecto_autoslug_field/issues/10)\n\n#","ref":"changelog.html#bugs","title":"Bugs - Changelog","type":"extras"},{"doc":"- Updates docs, fixes spelling","ref":"changelog.html#documentation","title":"Documentation - Changelog","type":"extras"},{"doc":"- Added `credo` support\n- Added `elixir` version 1.4 support","ref":"changelog.html#v0-2-1-2017-02-01","title":"v0.2.1 - 2017-02-01 - Changelog","type":"extras"},{"doc":"- Since this version only `ecto` 2 and above are supported\n- Updated docs on how to use this package with older `ecto`\n- Credo is not working with this release, please see https://github.com/sobolevn/ecto_autoslug_field/issues/5","ref":"changelog.html#v0-2-0-2016-11-02","title":"v0.2.0 - 2016-11-02 - Changelog","type":"extras"},{"doc":"- Now `build_slug/1` is called only when needed, changed docs appropriately\n- Replaced `cond` inside `do_generate_slug/3` with more obvious `if`\n- Changed tests structure, reached 100% coverage","ref":"changelog.html#v0-1-3-2016-07-22","title":"v0.1.3 - 2016-07-22 - Changelog","type":"extras"},{"doc":"- Moved `get_sources/2` and `build_slug/1` definitions into `SlugBase`\n- Added `","ref":"changelog.html#v0-1-2-2016-07-21","title":"v0.1.2 - 2016-07-21 - Changelog","type":"extras"},{"doc":"- Added new tests","ref":"changelog.html#options-and-function-section-to-the-readme-md","title":"Options` and `## Function` section to the `README.md` - Changelog","type":"extras"},{"doc":"- Since the docs inside `__using__` were unreachable the design of application was changed\n- Now `opts` keyword-list contains `:slug_build` parameter with a `build_slug` callback\n- Also `coverage` information is updated, since more code is reachable now","ref":"changelog.html#v0-1-1-2016-07-21","title":"v0.1.1 - 2016-07-21 - Changelog","type":"extras"},{"doc":"- Initial release","ref":"changelog.html#v0-1-0-2016-07-21","title":"v0.1.0 - 2016-07-21 - Changelog","type":"extras"},{"doc":"The MIT License (MIT)\n\nCopyright (c) 2016 Nikita Sobolev\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.","ref":"license.html","title":"License","type":"extras"},{"doc":"# EctoAutoslugField\n\n[![Build Status](https://github.com/sobolevn/ecto_autoslug_field/workflows/test/badge.svg?branch=master&event=push)](https://github.com/sobolevn/ecto_autoslug_field/actions?query=workflow%3Atest)\n[![Coverage Status](https://coveralls.io/repos/github/sobolevn/ecto_autoslug_field/badge.svg?branch=master)](https://coveralls.io/github/sobolevn/ecto_autoslug_field?branch=master)\n[![Module Version](https://img.shields.io/hexpm/v/ecto_autoslug_field.svg)](https://hex.pm/packages/ecto_autoslug_field)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/ecto_autoslug_field/)\n[![License](https://img.shields.io/hexpm/l/ecto_autoslug_field.svg)](https://github.com/sobolevn/ecto_autoslug_field/blob/master/LICENSE)\n\n`ecto_autoslug_field` is a reusable [`Ecto`](https://github.com/elixir-ecto/ecto) library which can automatically create slugs from other fields. We use [`slugify`](https://github.com/jayjun/slugify) as a default slug-engine.\n\nWe only depend on the `ecto` package (we do not deal with `ecto_sql` at all).\nWe support `ecto >= 3.7 and ecto < 4`!\n\nSee [this blog post](https://sobolevn.me/2017/07/creating-slugs-for-ecto-schemas)\nfor more information.","ref":"readme.html","title":"Overview","type":"extras"},{"doc":"```elixir\ndef deps do\n  [\n    {:ecto_autoslug_field, \"~> 3.1\"}\n  ]\nend\n```","ref":"readme.html#installation","title":"Installation - Overview","type":"extras"},{"doc":"There are several options to configure.\n\nRequired:\n\n- `:to` - represents the slug field name where to save value to\n\nOptional:\n\n- `:from` - represents the source fields from which to build slug, if this option is not set you have to override `get_sources/2` function\n- `:always_change` - if this option is set slug will be recreated from the given sources each time `maybe_generate_slug` function is called","ref":"readme.html#options","title":"Options - Overview","type":"extras"},{"doc":"- `get_sources/2` - this function is used to get sources for the slug, [docs](https://hexdocs.pm/ecto_autoslug_field/EctoAutoslugField.SlugBase.html#get_sources/2).\n- `build_slug/2` - this function is a place to modify the result slug, [docs](https://hexdocs.pm/ecto_autoslug_field/EctoAutoslugField.SlugBase.html#build_slug/2).","ref":"readme.html#functions","title":"Functions - Overview","type":"extras"},{"doc":"The simplest example:\n\n```elixir\ndefmodule EctoSlugs.Blog.Article.TitleSlug do\n  use EctoAutoslugField.Slug, from: :title, to: :slug\nend\n\ndefmodule EctoSlugs.Blog.Article do\n  use Ecto.Schema\n  import Ecto.Changeset\n  alias EctoSlugs.Blog.Article\n  alias EctoSlugs.Blog.Article.TitleSlug\n\n  schema \"blog_articles\" do\n    field :breaking, :boolean, default: false\n    field :content, :string\n    field :title, :string\n\n    field :slug, TitleSlug.Type\n\n    timestamps()\n  end\n\n  def changeset(model, params \\\\ :invalid) do\n    model\n    |> cast(params, [:title, :content, :breaking])\n    |> validate_required([:title, :content])\n    |> unique_constraint(:title)\n    |> TitleSlug.maybe_generate_slug()\n    |> TitleSlug.unique_constraint()\n  end\nend\n```\n\nSee [this tutorial](https://sobolevn.me/2017/07/creating-slugs-for-ecto-schemas)\nfor some more examples.","ref":"readme.html#examples","title":"Examples - Overview","type":"extras"},{"doc":"See [CHANGELOG.md](./CHANGELOG.md).","ref":"readme.html#changelog","title":"Changelog - Overview","type":"extras"},{"doc":"Copyright (c) 2016 Nikita Sobolev\n\nThis library is released under the MIT License. See the [LICENSE.md](./LICENSE.md) file\nfor further details.","ref":"readme.html#copyright-and-license","title":"Copyright and License - Overview","type":"extras"}]}